/* Example code for FT51 EVM module.
 *
 */
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "ft51.h"
#include "ft51_sfr.h"

#include "registers\ft51_registers.h"
#include "registers\ft51_aio_registers.h"
#include "registers\ft51_dio_registers.h"
#include "registers\ft51_spi_registers.h"
#include "registers\ft51_iocell_registers.h"
#include "registers\ft51_iomux_registers.h"
#include "registers\ft51_pwm_registers.h"

#include "ft51_general_config.h"
#include "ft51_interrupts.h"
#include "ft51_spim.h"

#include "ft51_usb.h"
#include "ft51_usb_dfu.h"
#include "ft51_dfu.h"

#include "ft51_iomux.h"
#include "ft51_i2c_master.h"
#include "ft51_lcd.h"

#define TRI_SCREEN

// LED #defines the LEDs are active low.
#ifdef TRI_SCREEN
#define RED_LED                 0x6f
#define GREEN_LED               0x5f
#define BLUE_LED                0x3f
#else
#define WHITE_LED               0x6f
#endif
#define LED_OFF                 0x7f
#define HB_LED                  0x01

// Time in milliseconds we need the button depressed for to count as a press.
#define BUTTON_CONFIDENCE 100
// Time in milliseconds for the USB latency.
#define LATENCY 16
// Time in milliseconds after which we sample the ADCs.
#define SAMPLE_TIME 1000

// Scale is clock frequency in MHz (assume 48) / clock-cycles per instruction.
// 16-bit timer counts up to 65536, so subtract duration from 65536.
#define MICROSECONDS_TO_TIMER_TICKS(a) (65536 - ((a) * (48 / 12)))

volatile unsigned short heartRateSampleTime = 0;

/**
 @name ms_timer
 @brief Count down ms timer.
 */
static volatile int16_t ms_timer = 0;

/**
 @name USB and Hub Configuration
 @brief Indication of whether the USB Hub is enabled, how it is
        powered and the size of the control endpoint Max Packets.
 */
//@{
// USB Hub Enabled - set to 1 for enabled and 0 for disabled
#define USB_HUB_ENABLED 0
// USB Bus Powered - set to 1 for self-powered or 0 for bus-powered
#define USB_SELF_POWERED 0
#if USB_SELF_POWERED == 1
#define USB_CONFIG_BMATTRIBUTES_VALUE (USB_CONFIG_BMATTRIBUTES_SELF_POWERED | USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1)
#else // USB_SELF_POWERED
#define USB_CONFIG_BMATTRIBUTES_VALUE USB_CONFIG_BMATTRIBUTES_RESERVED_SET_TO_1
#endif // USB_SELF_POWERED
// USB Endpoint Zero packet size (both must match)
#define USB_CONTROL_EP_MAX_PACKET_SIZE 8
#define USB_CONTROL_EP_SIZE USB_EP_SIZE_8
//@}

/**
 @name Device Configuration Areas
 @brief Size and location of string descriptors and
 structure used to modify device and configuration
 descriptors.
 */
//@{
// String descriptors - allow a maximum of 256 bytes for this
#define STRING_DESCRIPTOR_LOCATION 0x80
#define STRING_DESCRIPTOR_ALLOCATION 0x7A
//@}

/**
 @name DFU_BLOCK_SIZE definition.
 @brief Number of bytes in block, sent in each DFU_DNLOAD request
 from the DFU update program on the host. This is simplified
 in that the meaning of a block is an arbitrary number of
 bytes. This is intentionally a multiple of the maximum
 packet size for the control endpoints.
 */
//@{
#define DFU_BLOCK_SIZE 16

/**
 @name USB_PID_DEMO configuration.
 @brief Run Time Product ID for demo function.
 */
//@{
#define USB_PID_DEMO 0x0feb
//@}

/**
 @name DFU_USB_PID_DFUMODE configuration.
 @brief FTDI predefined DFU Mode Product ID.
 */
//@{
#define DFU_USB_PID_DFUMODE 0x0fee
//@}

/**
 @name DFU_USB_INTERFACE configuration..
 @brief Run Time and DFU Mode Interface Numbers.
 */
//@{
#define DFU_USB_INTERFACE_RUNTIME 1
#define DFU_USB_INTERFACE_DFUMODE 0
//@}

/**
 @name string_descriptor
 @brief Table of USB String descriptors

 This is placed at a fixed location in the __code section allowing
 up-to 256 bytes of string descriptors to be defined. These can be
 modified or replaced by a utility or binary editor without
 requiring a recompilation of the firmware.
 They are placed at offset 0x100 and reserve 0x100 bytes.
 The data is not stored in __xdata section and is therefore
 regarded as const.
 */
__code __at(STRING_DESCRIPTOR_LOCATION)uint8_t string_descriptor[STRING_DESCRIPTOR_ALLOCATION] =
{
    // String 0 is actually a list of language IDs supported by the real strings.
    0x04, USB_DESCRIPTOR_TYPE_STRING, 0x09,
    0x04, // 0409 = English (US)

    // String 1 (Manufacturer): "FTDI"
    0x0a, USB_DESCRIPTOR_TYPE_STRING, 'F', 0x00, 'T', 0x00, 'D', 0x00, 'I',
    0x00,

    // String 2 (Product): "FT51A Demo"
    0x1E, USB_DESCRIPTOR_TYPE_STRING, 'F', 0x00, 'T', 0x00, '5', 0x00, '1',
    0x00, 'A', 0x00, ' ', 0x00, 'D', 0x00, 'e', 0x00, 'm', 0x00, 'o', 0x00,

    // END OF STRINGS
    0x00
};

/**
 @name device_descriptor_demo
 @brief Device descriptor for Run Time mode.
 */
__code USB_device_descriptor device_descriptor_demo =
{
    .bLength = 0x12,
    .bDescriptorType = USB_DESCRIPTOR_TYPE_DEVICE,
    .bcdUSB = USB_BCD_VERSION_2_0,          // V2.0
    .bDeviceClass = USB_CLASS_DEVICE,       // Defined in interface
    .bDeviceSubClass = USB_SUBCLASS_DEVICE, // Defined in interface
    .bDeviceProtocol = USB_PROTOCOL_DEVICE, // Defined in interface
    .bMaxPacketSize0 = USB_CONTROL_EP_MAX_PACKET_SIZE,
    .idVendor = USB_VID_FTDI,   // idVendor: 0x0403 (FTDI)
    .idProduct = USB_PID_DEMO, // idProduct: 0x0feb
    .bcdDevice = 0x0101,        // 1.1
    .iManufacturer = 0x01,      // Manufacturer
    .iProduct = 0x02,           // Product
    .iSerialNumber = 0x03,      // Serial Number
    .bNumConfigurations = 0x01
};

/**
 @name config_descriptor_demo
 @brief Config descriptor for Run Time mode.
 */
//@{
__code struct config_descriptor_demo
{
    USB_configuration_descriptor configuration;
    USB_interface_descriptor interface;
    USB_endpoint_descriptor endpoint_in;
    USB_endpoint_descriptor endpoint_out;
    USB_interface_descriptor dfu_interface;
    USB_dfu_functional_descriptor dfu_functional;
};

// Fields marked with * are updated with information in device_config[]
struct config_descriptor_demo config_descriptor_demo =
{
    .configuration.bLength = 0x09,
    .configuration.bDescriptorType = USB_DESCRIPTOR_TYPE_CONFIGURATION,
    .configuration.wTotalLength = sizeof(struct config_descriptor_demo),
    .configuration.bNumInterfaces = 0x02,
    .configuration.bConfigurationValue = 0x01,
    .configuration.iConfiguration = 0x00,
    .configuration.bmAttributes = USB_CONFIG_BMATTRIBUTES_VALUE,
    .configuration.bMaxPower = 0xFA,           // 500mA
    // ---- INTERFACE DESCRIPTOR for Demo ----
    .interface.bLength = 0x09,
    .interface.bDescriptorType = USB_DESCRIPTOR_TYPE_INTERFACE,
    .interface.bInterfaceNumber = 0,
    .interface.bAlternateSetting = 0x00,
    .interface.bNumEndpoints = 0x02,
    .interface.bInterfaceClass = USB_CLASS_VENDOR, // bInterfaceClass: Vendor
    .interface.bInterfaceSubClass = 0xFF,       // bInterfaceSubClass:
    .interface.bInterfaceProtocol = 0xFF,       // bInterfaceProtocol:
    .interface.iInterface = 0x05,            // iInterface: "FT51A Demo"
    // ---- ENDPOINT DESCRIPTOR for Demo ----
    .endpoint_in.bLength = 0x07,
    .endpoint_in.bDescriptorType = USB_DESCRIPTOR_TYPE_ENDPOINT,
    .endpoint_in.bEndpointAddress = 0x01,
    .endpoint_in.bmAttributes = USB_ENDPOINT_DESCRIPTOR_ATTR_BULK,
    .endpoint_in.wMaxPacketSize = 0x0040,
    .endpoint_in.bInterval = 0x0,
    // ---- ENDPOINT DESCRIPTOR for Demo ----
    .endpoint_out.bLength = 0x07,
    .endpoint_out.bDescriptorType = USB_DESCRIPTOR_TYPE_ENDPOINT,
    .endpoint_out.bEndpointAddress = 0x81,
    .endpoint_out.bmAttributes = USB_ENDPOINT_DESCRIPTOR_ATTR_BULK,
    .endpoint_out.wMaxPacketSize = 0x0040,
    .endpoint_out.bInterval = 0x0,
    // ---- INTERFACE DESCRIPTOR for DFU Interface ----
    .dfu_interface.bLength = 0x09,
    .dfu_interface.bDescriptorType = USB_DESCRIPTOR_TYPE_INTERFACE,
    .dfu_interface.bInterfaceNumber = DFU_USB_INTERFACE_RUNTIME,
    .dfu_interface.bAlternateSetting = 0x00,
    .dfu_interface.bNumEndpoints = 0x00,
    .dfu_interface.bInterfaceClass = USB_CLASS_APPLICATION, // bInterfaceClass: Application Specific Class
    .dfu_interface.bInterfaceSubClass = USB_SUBCLASS_DFU, // bInterfaceSubClass: Device Firmware Update
    .dfu_interface.bInterfaceProtocol = USB_PROTOCOL_DFU_RUNTIME, // bInterfaceProtocol: Runtime Protocol
    .dfu_interface.iInterface = 0x05,       // * iInterface: "DFU Interface"
    // ---- FUNCTIONAL DESCRIPTOR for DFU Interface ----
    .dfu_functional.bLength = 0x09,
    .dfu_functional.bDescriptorType = USB_DESCRIPTOR_TYPE_DFU_FUNCTIONAL,
    .dfu_functional.bmAttributes = 0x01,  	// bmAttributes: bitCanDnload
    .dfu_functional.wDetatchTimeOut = DFU_TIMEOUT, // wDetatchTimeOut: suggest 8192ms
    .dfu_functional.wTransferSize = DFU_BLOCK_SIZE, // wTransferSize: make it 16 bytes
    .dfu_functional.bcdDfuVersion = USB_BCD_VERSION_DFU_1_1, // bcdDfuVersion: DFU Version 1.1
};
//@}

/**
 @name device_descriptor_dfumode
 @brief Device descriptor for DFU Mode.
 */
__code USB_device_descriptor device_descriptor_dfumode =
{
    .bLength = 0x12,
    .bDescriptorType = USB_DESCRIPTOR_TYPE_DEVICE,
    .bcdUSB = USB_BCD_VERSION_2_0,          // V2.00
    .bDeviceClass = USB_CLASS_DEVICE,       // Defined in interface
    .bDeviceSubClass = USB_SUBCLASS_DEVICE, // Defined in interface
    .bDeviceProtocol = USB_PROTOCOL_DEVICE, // Defined in interface
    .bMaxPacketSize0 = USB_CONTROL_EP_MAX_PACKET_SIZE,    // 8
    .idVendor = USB_VID_FTDI,   // idVendor: 0x0403 (FTDI)
    .idProduct = DFU_USB_PID_DFUMODE, // idProduct: 0x0fee
    .bcdDevice = 0x0101,        // 1.1
    .iManufacturer = 0x01,      // Manufacturer
    .iProduct = 0x04,           // Product
    .iSerialNumber = 0x03,      // Serial Number
    .bNumConfigurations = 0x01
};

/**
 @name config_descriptor_dfumode
 @brief Config descriptor for DFU Mode.
 */
//@{
__code struct config_descriptor_dfumode
{
    USB_configuration_descriptor configuration;
    USB_interface_descriptor dfu_interface;
    USB_dfu_functional_descriptor dfu_functional;
};

// Fields marked with * are updated with information in device_config[]
struct config_descriptor_dfumode config_descriptor_dfumode =
{
    .configuration.bLength = 0x09,
    .configuration.bDescriptorType = USB_DESCRIPTOR_TYPE_CONFIGURATION,
    .configuration.wTotalLength = sizeof(struct config_descriptor_dfumode),
    .configuration.bNumInterfaces = 0x01,
    .configuration.bConfigurationValue = 0x01,
    .configuration.iConfiguration = 0x00,
    .configuration.bmAttributes = USB_CONFIG_BMATTRIBUTES_VALUE,
    .configuration.bMaxPower = 0xFA, // 500mA

    // ---- INTERFACE DESCRIPTOR for DFU Interface ----
    .dfu_interface.bLength = 0x09,
    .dfu_interface.bDescriptorType = USB_DESCRIPTOR_TYPE_INTERFACE,
    .dfu_interface.bInterfaceNumber = DFU_USB_INTERFACE_DFUMODE,
    .dfu_interface.bAlternateSetting = 0x00,
    .dfu_interface.bNumEndpoints = 0x00,
    .dfu_interface.bInterfaceClass = USB_CLASS_APPLICATION, // bInterfaceClass: Application Specific Class
    .dfu_interface.bInterfaceSubClass = USB_SUBCLASS_DFU, // bInterfaceSubClass: Device Firmware Update
    .dfu_interface.bInterfaceProtocol = USB_PROTOCOL_DFU_DFUMODE, // bInterfaceProtocol: Runtime Protocol
    .dfu_interface.iInterface = 0x05,
    // ---- FUNCTIONAL DESCRIPTOR for DFU Interface ----
    .dfu_functional.bLength = 0x09,
    .dfu_functional.bDescriptorType = USB_DESCRIPTOR_TYPE_DFU_FUNCTIONAL,
    .dfu_functional.bmAttributes = 0x01,  	// bmAttributes: bitCanDnload
    .dfu_functional.wDetatchTimeOut = DFU_TIMEOUT, // wDetatchTimeOut: suggest 8192ms
    .dfu_functional.wTransferSize = DFU_BLOCK_SIZE,     // wTransferSize: 16
    .dfu_functional.bcdDfuVersion = USB_BCD_VERSION_DFU_1_1, // bcdDfuVersion: DFU Version 1.1
};
//@}

enum MENU_STATES
{
    TEMP = 0,
    FORCE,
    PULSE,
};

//-------------------------------------------------------------------------//
//                                  Globals                                //
//-------------------------------------------------------------------------//
// USB
uint8_t issueIns = FALSE;

// Sensors
static uint16_t force;
static uint16_t pulseSample;
static uint16_t temperature;

// Menu
static int8_t menuState = 0;
static uint8_t menuChanged;

// Buttons
uint16_t pressConfidence = 0;

// Timers
static uint16_t sampleCount;
static uint8_t latencyCount;
static uint16_t heartRateExpired;
static uint16_t sample;

//-------------------------------------------------------------------------//
//                               Menu Buttons                              //
//-------------------------------------------------------------------------//
void detect_button_press()
{
    if (((P0 & 0x02) == 0) || ((P0 & 0x04) == 0))
    {
        pressConfidence++;
        if (pressConfidence == BUTTON_CONFIDENCE)
        {
            if ((P0 & 0x02) == 0)
            {
                menuChanged = 1;
                ++menuState;
                if (menuState == 3)
                    menuState = 0;
            }
            else
            {
                menuChanged = 1;
                --menuState;
                if (menuState == -1)
                    menuState = 2;

            }
        }
    }
    else
    {
        pressConfidence = 0;
    }
}

//-------------------------------------------------------------------------//
//                              LATENCY TIMER                              //
//-------------------------------------------------------------------------//
// Invoked by ISR for Timer0 every millisecond
void latency_interrupt(const uint8_t flags)
{
	(void)flags; // Flags not currently used

    detect_button_press();

    latencyCount++;
//    if (((latencyCount | 0xFE) & 0xff) == 0xfe) // Pulse sample expired.
    if ((latencyCount % 2) == 0) // Pulse sample expired.
    {
        // ADC sample has completed so set the PWM comparator as appropriate.
        if (force > 50)
        {
            IO_REG_GENERAL_WRITE(PWM_CMP16_0_MSB, (force >> 8));
            IO_REG_GENERAL_WRITE(PWM_CMP16_0_LSB, (force & 0xfc));
        }
        else
        {
            IO_REG_GENERAL_WRITE(PWM_CMP16_0_MSB, 0x00);
            IO_REG_GENERAL_WRITE(PWM_CMP16_0_LSB, 0x00);
        }

        heartRateSampleTime += 2;
        heartRateExpired = TRUE;
    }

    // if ((sampleCount++ % 880) == 0) // USB Sample expired.
    if ((sampleCount++ % 240) == 0) // USB Sample expired.
    {
        sample = TRUE;
    }
    if ((sampleCount % 100) == 0) // USB Sample expired.
    {
        menuChanged = TRUE;
    }

    if (ms_timer)
    {
        ms_timer--;
    }

    // The DFU detach timer must be called once per millisecond
    dfu_timer();

    // Reload the timer
    TH0 = MSB(MICROSECONDS_TO_TIMER_TICKS(1000));
    TL0 = LSB(MICROSECONDS_TO_TIMER_TICKS(1000));
}

void latency_timer_initialise(void)
{
    // Register our own handler for interrupts from Timer 0
    (void)interrupts_register(latency_interrupt, interrupts_timer0);

    // Timer0 is controlled by TMOD bits 0 to 3, and TCON bits 4 to 5.
    TMOD &= 0xF0; // Clear Timer0 bits
    TMOD |= 0x01; // Put Timer0 in mode 1 (16 bit)

    // Set the count-up value so that it rolls over to 0 after 1 millisecond.
    TH0 = MSB(MICROSECONDS_TO_TIMER_TICKS(1000));
    TL0 = LSB(MICROSECONDS_TO_TIMER_TICKS(1000));

    TCON &= 0xCF; // Clear Timer0's Overflow and Run flags
    TCON |= 0x10; // Start Timer0 (set its Run flag)
}

//-------------------------------------------------------------------------//
//                                  USB                                    //
//-------------------------------------------------------------------------//
/**
 @brief      USB class request handler
 @details    Handle class requests from the host application.
 The bRequest value is parsed and the appropriate
 action or function is performed. Additional values
 from the USB_device_request structure are decoded
 and the DFU state machine and status updated as
 required.
 @param[in]	req - USB_device_request structure containing the
 SETUP portion of the request from the host.
 @return		status - FT51_OK if successful or FT51_FAILED
 if there is an error or the bRequest is not
 supported.
 **/
FT51_STATUS class_req_cb(USB_device_request *req)
{
    FT51_STATUS status = FT51_FAILED;
    uint8_t interface = LSB(req->wIndex) & 0x0F;

    // For DFU requests the SETUP packet must be addressed to the
    // the DFU interface on the device.

    // For DFU requests ensure the recipient is an interface...
    if ((req->bmRequestType & USB_BMREQUESTTYPE_RECIPIENT_MASK) ==
    USB_BMREQUESTTYPE_RECIPIENT_INTERFACE)
    {
        // ...and that the interface is the correct Runtime interface
        if (dfu_is_runtime())
        {
            if ((interface == DFU_USB_INTERFACE_RUNTIME))
            {
                status = FT51_OK;
                // Handle only DFU_DETATCH, DFU_GETSTATE and DFU_GETSTATUS
                // when in Runtime mode. Table 3.2 DFU_DETACH is mandatory
                // in Runtime mode, DFU_GETSTATE and DFU_GETSTATUS are
                // optional.
                switch (req->bRequest)
                {
                case USB_CLASS_REQUEST_DETACH:
                    dfu_class_req_detach(req->wValue);
                    break;
                case USB_CLASS_REQUEST_GETSTATUS:
                    dfu_class_req_getstatus();
                    break;
                case USB_CLASS_REQUEST_GETSTATE:
                    dfu_class_req_getstate();
                    break;
                default:
                    status = FT51_FAILED;
                }
            }
        }
        // ...or the correct DFU Mode interface
        else
        {
            if (interface == DFU_USB_INTERFACE_DFUMODE)
            {
                status = FT51_OK;
                // Handle remaining DFU class requests when in DFU Mode.
                // Table 3.2 DFU_DETACH is not supported in DFU Mode.
                switch (req->bRequest)
                {
                case USB_CLASS_REQUEST_DNLOAD:
                    /* For simplicity only program the maximum block size
                     * starting at an address which is an exact multiple
                     * of this. The block size is given in the configuration
                     * descriptor for the DFU interface.
                     */
                    dfu_class_req_download(req->wValue * DFU_BLOCK_SIZE,
                            req->wLength);
                    break;
                case USB_CLASS_REQUEST_GETSTATUS:
                    dfu_class_req_getstatus();
                    break;
                case USB_CLASS_REQUEST_GETSTATE:
                    dfu_class_req_getstate();
                    break;
                case USB_CLASS_REQUEST_CLRSTATUS:
                    dfu_class_req_clrstatus();
                    break;
                case USB_CLASS_REQUEST_ABORT:
                    dfu_class_req_abort();
                    break;

                default:
                case USB_CLASS_REQUEST_UPLOAD:
                    status = FT51_FAILED;
                    // Unknown or unsupported request.
                    break;
                }
            }
        }
    }

    return status;
}

/**
 @brief      USB standard request handler for GET_DESCRIPTOR
 @details    Handle standard GET_DESCRIPTOR requests from the host
 application.
 The hValue is parsed and the appropriate device,
 configuration or string descriptor is selected.
 For device and configuration descriptors the DFU
 state machine determines which descriptor to use.
 For string descriptors the lValue selects which
 string from the string_descriptors table to use.
 Strings are obtained from the __code section and
 typecast to a be compatible with __xdata storage.
 The string table is automatically traversed to find
 the correct string and the length is taken from the
 string descriptor.
 @param[in]	req - USB_device_request structure containing the
 SETUP portion of the request from the host.
 @return		status - FT51_OK if successful or FT51_FAILED
 if there is an error or the bmRequestType is not
 supported.
 **/
FT51_STATUS standard_req_get_descriptor(USB_device_request *req)
{
    uint8_t *src = NULL;
    //NOTE: the requested length is assumed to be less than 256 bytes.
    uint8_t length = req->wLength & 0x00ff;
    uint8_t hValue = req->wValue >> 8;
    uint8_t lValue = req->wValue & 0x00ff;
    uint8_t i;
    //NOTE: the requested string length is assumed to be less than 256 bytes.
    uint8_t slen;

    switch (hValue)
    {
    case USB_DESCRIPTOR_TYPE_DEVICE:

        if (dfu_is_runtime())
        {
            src = (char *) &device_descriptor_demo;
        }
        else
        {
            src = (char *) &device_descriptor_dfumode;
        }
        break;

    case USB_DESCRIPTOR_TYPE_CONFIGURATION:

        if (dfu_is_runtime())
        {
            src = (char *) &config_descriptor_demo;
            if (length != 9) // First 9 bytes describe basic config.
                length = sizeof(config_descriptor_demo); // Entire structure.
        }
        else
        {
            src = (char *) &config_descriptor_dfumode;
            if (length != 9) // First 9 bytes describe basic config.
                length = sizeof(config_descriptor_dfumode); // Entire structure.
        }
        break;

    case USB_DESCRIPTOR_TYPE_STRING:

        // Find the nth string in the string descriptor table
        i = 0;
        while ((slen = string_descriptor[i]) > 0)
        {
            // Point to start of string descriptor in __code segment
            // Typecast prevents warning for losing const when usb_transfer
            // is called
            src = (uint8_t *) &string_descriptor[i];
            if (lValue == 0)
            {
                break;
            }
            i += slen;
            lValue--;
        }
        if (lValue > 0)
        {
            // String not found
            return FT51_FAILED;
        }
        // Update the length returned only if it is less than the requested
        // size
        if (length > slen)
        {
            length = slen;
        }

        break;

    default:
        return FT51_FAILED;
    }

    USB_transfer(USB_EP_0, USB_DIR_IN, src, length);

    return FT51_OK;
}

/**
 @brief      USB standard request handler for GET_STATUS
 @details    Handle standard GET_STATUS requests from the host
             application.
             The bmRequestType value is parsed and the appropriate
             action or function is performed.
 @param[in]	 req - USB_device_request structure containing the
             SETUP portion of the request from the host.
 @return	 status - FT51_OK if successful or FT51_FAILED
             if there is an error or the bmRequestType is not
             supported.
 **/
FT51_STATUS standard_req_get_status(USB_device_request *req)
{
    USB_STATE state = USB_get_state();
    USB_ENDPOINT_NUMBER ep_number = LSB(req->wIndex) & 0x0F;
    USB_ENDPOINT_DIR ep_dir = (LSB(req->wIndex) >> 7);
    uint8_t buf[2];
    uint8_t ep_index  = (ep_number * 2) + ep_dir;

    buf[0] = buf[1] = 0;

    // Get Status for endpoints, only halt status is needed
    if (req->bmRequestType == (USB_BMREQUESTTYPE_DIR_DEV_TO_HOST |
            USB_BMREQUESTTYPE_RECIPIENT_ENDPOINT))
    {
        if (((state < CONFIGURED) && (ep_number == 0))
                || (state >= CONFIGURED))
        {
            if (USB_get_ep_stalled(ep_number, ep_dir))
            {
                buf[0] = USB_GET_STATUS_ENDPOINT_HALT;
            }
            goto retOK;
        }
    }
    else if (req->bmRequestType == (USB_BMREQUESTTYPE_DIR_DEV_TO_HOST |
            USB_BMREQUESTTYPE_RECIPIENT_DEVICE))
    {
        // This will return the self-powered status (currently fixed)
        buf[0] = USB_GET_STATUS_DEVICE_SELF_POWERED;
        goto retOK;
    }
    if (req->bmRequestType == (USB_BMREQUESTTYPE_DIR_DEV_TO_HOST |
            USB_BMREQUESTTYPE_RECIPIENT_INTERFACE))
    {
        // Must return 2 bytes of zeros for an interface
        goto retOK;
    }
    return FT51_FAILED;

retOK:
    USB_transfer(USB_EP_0, USB_DIR_IN, buf, 2);

    return FT51_OK;
}

/**
 @brief      USB standard request handler for CLEAR_FEATURE and SET_FEATURE
 @details    Handle standard CLEAR_FEATURE and SET_FEATURE requests
             from the host application.
             The bmRequestType value is parsed and the appropriate
             action or function is performed.
 @param[in]  req - USB_device_request structure containing the
             SETUP portion of the request from the host.
 @return        status - FT51_OK if successful or FT51_FAILED
             if there is an error or the bmRequestType is not
             supported.
 **/
FT51_STATUS standard_req_set_clear_feature(USB_device_request *req)
{
    USB_ENDPOINT_NUMBER ep_number = LSB(req->wIndex) & 0x0F;
    USB_ENDPOINT_DIR ep_dir = (LSB(req->wIndex) >> 7);
    USB_STATE state = USB_get_state();

    if (req->bmRequestType == (USB_BMREQUESTTYPE_DIR_HOST_TO_DEV |
            USB_BMREQUESTTYPE_RECIPIENT_ENDPOINT))
    {
        // Only support the endpoint halt feature in this device
        if (req->wValue == USB_FEATURE_ENDPOINT_HALT)
        {
            // Only allow this if the device is configured or the endpoint
            // is zero if the device is not configured.
            if (((state < CONFIGURED) && (ep_number == 0))
                    || (state >= CONFIGURED))
            {
                // Perform a stall or a clear
                if (req->bRequest == USB_REQUEST_CODE_CLEAR_FEATURE)
                {
                    // Feature selector CLEAR
                    USB_clear_endpoint(ep_number, ep_dir);
                }
                else
                {
                    // Feature selector SET
                    USB_stall_endpoint(ep_number, ep_dir);
                }

                USB_transfer(USB_EP_0, USB_DIR_IN, NULL, 0);
                return FT51_OK;
            }
        }
    }
    return FT51_FAILED;
}

/**
 @brief      USB standard request handler
 @details    Handle standard requests from the host application.
 The bRequest value is parsed and the appropriate
 action or function is performed. Additional values
 from the USB_device_request structure are decoded
 and provided to other handlers.
 @param[in]	req - USB_device_request structure containing the
 SETUP portion of the request from the host.
 @return		status - FT51_OK if successful or FT51_FAILED
 if there is an error or the bRequest is not
 supported.
 **/
FT51_STATUS standard_req_cb(USB_device_request *req)
{
    FT51_STATUS status = FT51_FAILED;

    switch (req->bRequest)
    {
    case USB_REQUEST_CODE_GET_STATUS:
        status = standard_req_get_status(req);
        break;

    case USB_REQUEST_CODE_CLEAR_FEATURE:
    case USB_REQUEST_CODE_SET_FEATURE:
        status = standard_req_set_clear_feature(req);
        break;

    case USB_REQUEST_CODE_GET_DESCRIPTOR:
        status = standard_req_get_descriptor(req);
        break;

    case USB_REQUEST_CODE_SET_CONFIGURATION:
        status = USB_req_set_configuration(req);
        break;

    case USB_REQUEST_CODE_GET_CONFIGURATION:
        status = USB_req_get_configuration();
        break;

    case USB_REQUEST_CODE_SET_ADDRESS:
        status =  USB_req_set_address(req);
        break;

    default:
    case USB_REQUEST_CODE_GET_INTERFACE:
    case USB_REQUEST_CODE_SET_INTERFACE:
        // Unknown or unsupported request.
        status = FT51_FAILED;
        break;
    }

    return status;
}

/**
 @brief      USB vendor request handler
 @details    Handle vendor requests from the host application.
 The bRequest value is parsed and the appropriate
 action or function is performed. Additional values
 from the USB_device_request structure are decoded
 and provided to other handlers.
 There are no vendor requests requiring handling in
 this firmware.
 @param[in]	req - USB_device_request structure containing the
 SETUP portion of the request from the host.
 @return		status - FT51_OK if successful or FT51_FAILED
 if there is an error or the bRequest is not
 supported.
 **/
FT51_STATUS vendor_req_cb(USB_device_request *req)
{
    (void) req;
    return FT51_FAILED;
}

/**
 @brief      USB reset callback
 @details    Called when the USB bus enters the reset state.
 @param[in]	status - unused.
 @return		N/A
 **/
void reset_cb(uint8_t status)
{
    (void) status;

    USB_free_endpoint(USB_EP_1, USB_DIR_OUT);
    USB_free_endpoint(USB_EP_1, USB_DIR_IN);

    if (dfu_reset())
    {
        // Run downloaded application - reload MTP to shadow RAM
        // Execution effectively stops now and returns to the main()
        // function of the new program in MTP.
        device_revert();
    }

    /* Make application specific BULK, INTERRUPT or
     * ISOCHRONOUS endpoints.
     */
    USB_create_endpoint(USB_EP_1, USB_EP_BULK, USB_DIR_OUT, USB_EP_SIZE_64);
    USB_create_endpoint(USB_EP_1, USB_EP_BULK, USB_DIR_IN, USB_EP_SIZE_64);
    return;
}

/**
 @brief      Initialise emulation of a USB device
 @details    Sets up callback functions and data structures
 required to implement control endpoints on device.
 @param[in]	ep_size - Endpoint size of control endpoints.
 @return     N/A
 **/
void usb_setup(void)
{
    USB_ctx usb_ctx;
    memset(&usb_ctx, 0, sizeof(usb_ctx));

    usb_ctx.standard_req = standard_req_cb;
    usb_ctx.class_req = class_req_cb;
    usb_ctx.vendor_req = vendor_req_cb;
    usb_ctx.suspend = NULL;
    usb_ctx.resume =NULL;
    usb_ctx.reset = reset_cb;
    usb_ctx.lpm = NULL;

    // Hub disabled
    usb_ctx.hub_en.mask.hub_enable = USB_HUB_ENABLED;
    usb_ctx.hub_en.mask.hub_LocalPwrSrc = USB_SELF_POWERED;
    usb_ctx.hub_en.mask.hub_RmtWkUpEn = 1;

    // Initialise the USB device with a control endpoint size
    // of 8 bytes. This must match the size for bMaxPacketSize0
    // defined in the device descriptor.
    usb_ctx.ep_size = USB_CONTROL_EP_SIZE;

    // Perform USB Device initialisation.
    USB_initialise(&usb_ctx);

    return;
}

//-------------------------------------------------------------------------//
//                               IOMUX Setup                               //
//-------------------------------------------------------------------------//
void iomux_setup()
{
    // SPI Master
    IOMUX_OUTPUT(DIO_6, IOMUX_OUT_SPI_MASTER_SS_N_0);
    IOMUX_INPUT(DIO_4, IOMUX_IN_SPI_MASTER_MISO);
    IOMUX_OUTPUT(DIO_3, IOMUX_OUT_SPI_MASTER_MOSI);
    IOMUX_OUTPUT(DIO_5, IOMUX_OUT_SPI_MASTER_SCLK);

    IOMUX_set_pad_config(DIO_6,
                        IOMUX_PAD_DRIVE_CURRENT_16MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_4,
                        IOMUX_PAD_DRIVE_CURRENT_4MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_3,
                        IOMUX_PAD_DRIVE_CURRENT_16MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_5,
                        IOMUX_PAD_DRIVE_CURRENT_16MA,
                        IOMUX_PAD_TRIGGER_NORMAL,
                        IOMUX_PAD_SLEW_RATE_FAST,
                        IOMUX_PAD_PULL_NONE);

    // I2C Master
    IOMUX_OUTPUT(DIO_8, IOMUX_OUT_PORT0_0);
    IOMUX_OUTPUT(DIO_13, IOMUX_OUT_I2C_MASTER_SCL);
    IOMUX_OUTPUT(DIO_14, IOMUX_OUT_I2C_MASTER_SDA);
    IOMUX_INPUT (DIO_14, IOMUX_IN_I2C_MASTER_SDA);

    IOMUX_set_pad_config(DIO_8,IOMUX_PAD_DRIVE_CURRENT_16MA,
                            IOMUX_PAD_TRIGGER_SCHMITT,
                            IOMUX_PAD_SLEW_RATE_FAST,
                            IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_13,IOMUX_PAD_DRIVE_CURRENT_8MA,
                            IOMUX_PAD_TRIGGER_SCHMITT,
                            IOMUX_PAD_SLEW_RATE_FAST,
                            IOMUX_PAD_PULL_NONE);
    IOMUX_set_pad_config(DIO_14,IOMUX_PAD_DRIVE_CURRENT_4MA,
                            IOMUX_PAD_TRIGGER_SCHMITT,
                            IOMUX_PAD_SLEW_RATE_FAST,
                            IOMUX_PAD_PULL_NONE);

    // Buttons
    IOMUX_INPUT(DIO_0, IOMUX_IN_PORT0_1); // SW1
    IOMUX_INPUT(DIO_7, IOMUX_IN_PORT0_2); // SW2

    // HB LED
    IOMUX_OUTPUT(AIO_11, IOMUX_OUT_PORT3_0); /* Heartbeat LED */

    // LCD GPIO Leds
    IOMUX_OUTPUT(AIO_12, IOMUX_OUT_PORT2_4);
    IOMUX_OUTPUT(AIO_13, IOMUX_OUT_PORT2_5);
    IOMUX_OUTPUT(AIO_14, IOMUX_OUT_PORT2_6);

    // FSR PWM LED
    IOMUX_OUTPUT(DIO_1, IOMUX_OUT_PWM_OUT_0);

}

//-------------------------------------------------------------------------//
//                               I2C Master                                //
//-------------------------------------------------------------------------//

/**
 @name ADT7310 temperature sensor register addresses
 @brief Register addresses used to access setpoints, readings and status
        in the ADT7310 device.
 */
enum {
    ADT7310_REG_STATUS = 0x0,
    ADT7310_REG_CONFIGURATION = 0x1,
    ADT7310_REG_TEMPERATURE = 0x2,
    ADT7310_REG_ID = 0x3,
    ADT7310_REG_TCRIT = 0x4,
    ADT7310_REG_THYST = 0x5,
    ADT7310_REG_THIGH = 0x6,
    ADT7310_REG_TLOW = 0x7,
};

/**
 @brief Structure for command byte of the ADT7310 device.
 */
struct adt7310_cmd_byte {
    uint8_t mbz:2; // Must be zero.
    uint8_t cont:1; // Activates a continuous read mode on the temperature value register only.
    uint8_t addr:3; // Target register address.
    uint8_t rw:1; // Read/write bit, where 1 indicates a read and 0 indicates a write.
    uint8_t start:1; // Set to 0 to begin a bus transaction.
};

/**
 @brief      Reset ADT7310 temperature sensor
 @details    Sends a reset command to the ADT7310 temperature sensor using the
             SPI Master peripheral.
             Resets the contents of all registers to their power-on values.
             Following a reset, the user should allow a period of 500 us
             before addressing the SPI interface.
 **/
void temperature_reset()
{
    uint8_t     src[4] = {0xFF, 0xFF, 0xFF, 0xFF};
    uint8_t     dummy[4] = {0, 0, 0, 0};

    // A Logic 1 must be written to the ADT7310 line for at least 32
    // serial clock cycles to reset the serial interface.
    SPIM_SS_ON;
    SPIM_transceive_ints(src,
                         dummy, //dummy response
                         4);
    SPIM_SS_OFF;
}

/**
 @brief      Read ADT7310 temperature sensor
 @details    Sends a read command to the ADT7310 temperature sensor using the
             SPI Master peripheral.
 **/
uint16_t temperature_read()
{
    struct adt7310_cmd_byte *cmd_byte;
    uint8_t  src[3] = {0, 0, 0}; // command byte
    uint8_t  dest[3] = {0, 0, 0};

    SPIM_SS_ON;
    cmd_byte = (struct adt7310_cmd_byte*)&src[0];

    cmd_byte->addr = ADT7310_REG_TEMPERATURE;
    cmd_byte->rw = 1; // For write
    cmd_byte->cont = 0; // Continuous read

    SPIM_transceive_ints(src,
                        dest,
                        3);
    SPIM_SS_OFF;

    // The result is in 13-bit mode and the conversion is in bits
    // 15:3 of the word received after the command is sent (MSB first).
    // The lower 4 bits of the conversion are fractions of a degree
    // so they can be removed as well. Final degrees Celcius is
    // data word returned shifted right 7.
    return (((dest[1] << 8) | dest[2]) >> 7);
}

//-------------------------------------------------------------------------//
//                                  ADC ISR                                //
//-------------------------------------------------------------------------//
void adc_interrupt(const uint8_t flags)
{
    uint8_t interrupt = 0;
    uint8_t sample_l = 0x00;
    uint8_t sample_h = 0x00;

    (void)flags;

    DISABLE_INTERRUPTS;

    // Determine the interrupt source.
    IO_REG_GENERAL_READ(AIO_INTERRUPTS_0_7, interrupt);
    // FSR Output
    if (interrupt & MASK_AIO_0_INT)
    {
        IO_REG_GENERAL_READ(AIO_0_ADC_DATA_L_1, sample_l);
        IO_REG_GENERAL_READ(AIO_0_ADC_DATA_U_1, sample_h);

        force = ((sample_h << 8) | sample_l);

        // Clear ADC interrupt register bit
        IO_REG_GENERAL_WRITE(AIO_INTERRUPTS_0_7, MASK_AIO_0_INT);
    }

    // Determine the interrupt source.
    IO_REG_GENERAL_READ(AIO_INTERRUPTS_8_15, interrupt);
    // Pulse Rate
    if (interrupt & MASK_AIO_10_INT)
    {
    	IO_REG_GENERAL_READ(AIO_10_ADC_DATA_L_1, sample_l);
    	IO_REG_GENERAL_READ(AIO_10_ADC_DATA_U_1, sample_h);
        pulseSample = ((sample_h << 8) | sample_l);

        // Clear ADC interrupt register bit
        IO_REG_GENERAL_WRITE(AIO_INTERRUPTS_8_15, MASK_AIO_10_INT);
    }

    ENABLE_INTERRUPTS;
}

uint8_t GetDecimalPlace(uint8_t byte)
{
    switch (byte & 0x03)
    {
    case 0x01:
        return 25;
    case 0x02:
        return 5;
    case 0x03:
        return 75;
    default:
        return 0;
    }
}

//-------------------------------------------------------------------------//
//                                  Main                                   //
//-------------------------------------------------------------------------//
void main(void)
{
    // static uint16_t count = 0;
    uint8_t line2[30];
    uint16_t heartRateSamples[10];
    uint8_t heartRateSampleCount = 0; // used to determine pulse timing
    volatile unsigned short lastBeatTime = 0;  // used to find the inter beat interval
    uint8_t thresh = 50; // used to find instant moment of heart beat
    volatile uint8_t pulse = FALSE;
    uint8_t heartRate = 0; // stores the pulse rate in bpm.
    uint8_t trashSamples = 0;
    uint8_t heartRateCalculating = FALSE;

    uint8_t fsrForce = 0;

    device_initialise();

    iomux_setup();

    usb_setup();

    latency_timer_initialise();

    I2C_master_initialise();

    // Reset the LCD display
    P0 = 0xf8; // Set reset line of LCD high for 5ms
    ms_timer = 5;
    while (ms_timer);
    P0 = 0xf9; // Clear reset line of LCD

    // Wait 40ms after reset
    ms_timer = 40;
    while (ms_timer);

    // Initialise the display
    lcd_initialise();
    // Clear the display
    lcd_clear();

    // Wait for 2ms to clear to complete
    ms_timer = 2;
    while (ms_timer);
    lcd_display("FT51 Demo", 10);

    // Enable Force and Pulse ADCs.
    // Enable AIO Cell Controller
    IO_REG_GENERAL_WRITE(AIO_CONTROL, MASK_AIO_DEV_EN);
    // Setup AIO_0 to ADC
    IO_REG_GENERAL_WRITE(AIO_MODE_0, MASK_AIO_0_MODE_1);
    // Setup AIO_8 to ADC
    IO_REG_GENERAL_WRITE(AIO_MODE_2, MASK_AIO_10_MODE_1);
    // Enable AIO_0 interrupt
    IO_REG_GENERAL_WRITE(AIO_INTERRUPT_ENABLES_0_7, MASK_AIO_0_IEN);
    // Enable AIO_8 interrupt
    IO_REG_GENERAL_WRITE(AIO_INTERRUPT_ENABLES_8_15, MASK_AIO_10_IEN);
    // Setup AIO_2 to disable pull up/down and drive strength to zero
    IO_REG_GENERAL_WRITE(DIGITAL_CONTROL_AIO_0, 0x00);
    // Setup AIO_3 to disable pull up/down and drive strength to zero
    IO_REG_GENERAL_WRITE(DIGITAL_CONTROL_AIO_10, 0x00);
    // Register for ADC interrupts
    interrupts_register(adc_interrupt, interrupts_IO_cell);

    // Setup PWM for ADC output.
    // Top level PWM soft reset
    IO_REG_GENERAL_WRITE(PWM_CONTROL, MASK_PWM_SOFT_RESET);
    // Top level PWM enable
    IO_REG_GENERAL_WRITE(PWM_CONTROL, MASK_PWM_DEV_EN);
    /* Enable interrupt and output */
    IO_REG_GENERAL_WRITE(PWM_INT_CTRL,  MASK_PWM_EN);
    // Set initial output state
    IO_REG_GENERAL_WRITE(PWM_INIT,  0x00);
    // Set the initial state after each cycle.
    IO_REG_GENERAL_WRITE(PWM_OUT_CLR_EN, 0x01);
    /* Set prescaler */
    IO_REG_GENERAL_WRITE(PWM_PRESCALER, 0xff);
    /* Set counter CNT16 value */
    IO_REG_GENERAL_WRITE(PWM_CNT16_MSB, 0x04);
    IO_REG_GENERAL_WRITE(PWM_CNT16_LSB, 0x00);
    // Turn on comparator 0 for PWM_0.
    IO_REG_GENERAL_WRITE(PWM_OUT_TOGGLE_EN_0, 0x01);

    // Configure the SPI Master
    SPIM_initialise_ints();
    // SPI Master Temp Sensor
    SPIM_configure_slave(IOMUX_OUT_SPI_MASTER_SS_N_0, // slave 0
    					 1, // idle state
                         SPI_BIT_ORDER_MSB, // send MSB first
                         48, // (48 MHz clock / 2) / 48 == 500 kHz SCLK
                         SPI_MODE_0, // CPOL=0, CPHA=0
                         0); // no delay between SS and SCLK

    // Wait for 2ms to setup to complete
    ms_timer = 2;
    while (ms_timer);

    temperature_reset();

    P3 = 0x01;

    do // Main Loop.
    {
        USB_process();
        if (!dfu_is_runtime())
        {
            continue;
        }

        if (menuChanged)
        {
            // LCD Display
            lcd_position(1, 0);
            switch (menuState)
            {
            case TEMP:
                sprintf (line2, "Temperature: %d%c C     ", temperature, 0xDF);
                if (temperature >= 30)// Red, too high!
#ifdef TRI_SCREEN
                	P2 = RED_LED;
                else if (temperature < 27)// Blue, too low!
                	P2 = BLUE_LED;
                else // Okay!
                    P2 = GREEN_LED;
#else
                	P0 = WHITE_LED;
                else // Okay!
                	P0 = LED_OFF;

#endif
                break;
            case FORCE:
                fsrForce = force / 40;
                sprintf (line2, "Force: %d.%d N       ", fsrForce >> 2, GetDecimalPlace(fsrForce));
                if ((fsrForce >> 2) > 3) // Red, too high!
#ifdef TRI_SCREEN
                	P2 = RED_LED;
                else if ((fsrForce >> 2) < 1)
                	P2 = GREEN_LED;
                else
                	P2 = BLUE_LED;
#else
            		P0 = WHITE_LED;
            	else // Okay!
            		P0 = LED_OFF;
#endif
                break;
            case PULSE:
                lcd_display("Heart Rate:", 11);
                if (heartRate == 0 && !heartRateCalculating)
                    sprintf (line2, "-        ");
                else if (heartRateCalculating)
                    sprintf (line2, "Calculating...");
                else
                    sprintf (line2, "%dbpm    ", heartRate);
                if (heartRate >= 90)
#ifdef TRI_SCREEN
                	P2 = RED_LED;
                else if (heartRate == 0)
                	P2 = LED_OFF;
                else if (heartRate < 70)
                	P2 = GREEN_LED;
                else // Zero
                	P2 = BLUE_LED;
#else
            		P0 = WHITE_LED;
            	else // Okay!
            		P0 = LED_OFF;
#endif
                break;
            }

            lcd_display(line2, 20);

            menuChanged = FALSE;
        }

        if (sample) // After 1sec record the current state of the samples.
        {
            uint8_t i = 0;
            volatile uint8_t pressure = 0;

            // heart rate
            if (heartRateSampleCount >= 5) // Work out the average for the heart rate readings.
            {
                uint16_t total = 0;

                heartRateCalculating = FALSE;

                if (heartRateSampleCount >= 10)
                    heartRateSampleCount = 10;

                for ( ; i < heartRateSampleCount; i++)
                    total += heartRateSamples[i];
                heartRate = 60000u / (total / heartRateSampleCount);
            }
            else if (heartRateSampleCount > 0)
            {
                heartRateCalculating = TRUE;
            }

            // temp
            temperature = temperature_read();
            //ms_timer = 2;
            temperature_reset();
            ms_timer = 2;

            sample = FALSE;
            menuChanged = TRUE;
        }

        if (heartRateExpired)
        {
            volatile uint16_t signal;
            static uint8_t i;

            DISABLE_INTERRUPTS;
            signal = pulseSample;
            ENABLE_INTERRUPTS;

            if ((heartRateSampleTime - lastBeatTime) > 400)
            {
                if (signal > thresh && !pulse)
                {
                    pulse = TRUE;
                    P3 &= ~(0x01); // LED ON
                }

                if (signal < 10 && pulse)
                {
                    if (lastBeatTime != 0)
                    {

                        if (trashSamples >= 3) // Discard the first 3 samples.
                        {
                            heartRateSamples[i++] = heartRateSampleTime - lastBeatTime;
                            i %= 10;
                            ++heartRateSampleCount;
                        }

                        ++trashSamples;
                    }

                    lastBeatTime = heartRateSampleTime;

                    P3 |= 0x01; // LED OFF
                    pulse = FALSE;
                }

                if (lastBeatTime && ((heartRateSampleTime - lastBeatTime) > 2500))
                {
                    lastBeatTime = 0;
                    heartRateSampleTime = 0;
                    heartRateSampleCount = 0;
                    heartRate = 0;
                    trashSamples = 0;
                    heartRateCalculating = FALSE;
                    P3 |= 0x01; // LED OFF
                    i = 0;
                }
            }

            heartRateExpired = FALSE;
        }

        // Initiate ADC sample AIO_0 and AIO_10
        IO_REG_GENERAL_WRITE(AIO_SAMPLE_0, MASK_AIO_0_SAMPLE);
        IO_REG_GENERAL_WRITE(AIO_SAMPLE_1, MASK_AIO_10_SAMPLE);

    } while (1); // End Main Loop.
}
